'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

(function (Action) {
  Action["PUSH"] = "push";
  Action["BACK"] = "back";
  Action["FORWARD"] = "forward";
  Action["RESET"] = "reset";
})(exports.Action || (exports.Action = {}));

var createHistoryReducer = function createHistoryReducer(_ref) {
  var _ref$maxSize = _ref.maxSize,
      maxSize = _ref$maxSize === void 0 ? Number.MAX_SAFE_INTEGER : _ref$maxSize;
  return function (prevState, action) {
    var present = prevState.present,
        past = prevState.past,
        future = prevState.future;
    var hasPast = past.length > 0;
    var hasFuture = future.length > 0;
    var hasPresent = present !== null;

    switch (action.type) {
      case exports.Action.PUSH:
        return {
          present: action.state,
          past: hasPresent ? [].concat(past.slice(-maxSize), [present]) : past,
          future: []
        };

      case exports.Action.BACK:
        return {
          present: hasPast ? past[past.length - 1] : present,
          past: past.slice(0, past.length - 1),
          future: hasPresent && hasPast ? [].concat(future.slice(0, maxSize), [present]) : future
        };

      case exports.Action.FORWARD:
        return {
          present: hasFuture ? future[future.length - 1] : present,
          past: hasPresent && hasFuture ? [].concat(past.slice(-maxSize), [present]) : past,
          future: future.slice(0, future.length - 1)
        };

      case exports.Action.RESET:
        return {
          present: action.state || null,
          past: [],
          future: []
        };

      default:
        return prevState;
    }
  };
};

function useHistoryReducer(initialValue, maxSize) {
  if (maxSize === void 0) {
    maxSize = Number.MAX_SAFE_INTEGER;
  }

  react.useDebugValue({
    maxSize: maxSize,
    initialValue: initialValue
  });
  return react.useReducer(createHistoryReducer({
    maxSize: maxSize
  }), {
    past: [],
    future: [],
    present: initialValue
  });
}

var useUndoRedoHistory = function useUndoRedoHistory(dispatch) {
  var back = react.useCallback(function () {
    return dispatch({
      type: exports.Action.BACK
    });
  }, [dispatch]);
  var forward = react.useCallback(function () {
    return dispatch({
      type: exports.Action.FORWARD
    });
  }, [dispatch]);
  var onKeyDown = react.useCallback(function (event) {
    var isModifierKey = event.metaKey || event.ctrlKey;
    var isZKey = event.keyCode === 90;

    if (isZKey && isModifierKey && event.shiftKey) {
      forward();
    } else if (isZKey && isModifierKey) {
      back();
    }
  }, [back, forward]);
  var ref = react.useRef(onKeyDown);
  react.useEffect(function () {
    var type = 'keydown';
    var target = window;

    if (ref) {
      target.removeEventListener(type, ref.current);
    }

    ref.current = onKeyDown;
    target.addEventListener(type, ref.current);
    return function () {
      return target.removeEventListener(type, ref.current);
    };
  }, [onKeyDown]);
};

exports.createHistoryReducer = createHistoryReducer;
exports.useHistoryReducer = useHistoryReducer;
exports.useUndoRedoHistory = useUndoRedoHistory;
//# sourceMappingURL=history-reducer.cjs.development.js.map
